// ***************************************
//            Absolutelyautomation.com
//
//  Panel mount 7 segment display
//  with embedded Arduino
//  Working as serial slave
//
//****************************************


#include <SevenSeg.h> 
#include <Wire.h>
#include <EEPROM.h>

 
// Pin definition for each segment
int sega = 12;
int segb = 10;
int segc = 7;
int segd = 5;
int sege = 3;
int segf = 11;
int segg = 2;
int segp = 6;

// Pin definition for each common
int digit1 = 13;
int digit2 = A0;
int digit3 = A1;

String dispvalue = "------";

int FWMajorVersion=0;
int FWMinorVersion=1;

#define MAX_I2C_BUFF     20
#define MAX_CHARS        6

#define EEPROM_I2C_ADD   0x00
#define I2C_ADD_DEFAULT  0x08
#define CMD_CHNGADD      0x25
#define CMD_BLNKRATE     0x26
#define CMD_DISPTEXT     0x27

SevenSeg disp( sega, segb, segc, segd, sege, segf, segg );
const int numOfDigits = 3;
int digitPins[numOfDigits] = { digit1, digit2, digit3  };  

int firstboot;
int i2cadd;
int i2c_buffer[MAX_I2C_BUFF];
int i2c_bytesrcvd;
int i2c_rcvdone;
int blinkrate=0;
int blinkstate;
long previousMillis = 0;

String astring = "A00";    
String fstring = "F00";    

void setup() {                

  // display setup
  
  pinMode(sega, OUTPUT);     
  pinMode(segb, OUTPUT);     
  pinMode(segc, OUTPUT);     
  pinMode(segd, OUTPUT);     
  pinMode(sege, OUTPUT);     
  pinMode(segf, OUTPUT);     
  pinMode(segg, OUTPUT);     
  pinMode(segp, OUTPUT);     
  
  pinMode(digit1, OUTPUT);     
  pinMode(digit2, OUTPUT);     
  pinMode(digit3, OUTPUT);     


  pinMode(13, OUTPUT);     

  disp.setDPPin(segp);  
  //disp.setCommonCathode();
  disp.setCommonAnode();
  disp.setDigitPins( numOfDigits, digitPins );
  disp.setRefreshRate(300);
  disp.setTimer(1);  // Don't use with servo functions at the same time!
  //disp.setTimer(2);  // Don't use with tone() function at the same time!
  disp.startTimer();   
 
 // I2C slave setup
 
  i2cadd = EEPROM.read(EEPROM_I2C_ADD);
  if(i2cadd == 0xFF)
  {    
    EEPROM.write(EEPROM_I2C_ADD, I2C_ADD_DEFAULT); // EEPROM address erased, writting default value
    i2cadd = EEPROM.read(EEPROM_I2C_ADD);
  }  
  Wire.begin(i2cadd); // join I2C bus using the address stored in  EEPROM       
  Wire.onReceive(receiveEvent); // event register
      
}

// infinite loop:
void loop() {
  if(!firstboot){
    disp.write("8.8.8.");
    delay(500);
    disp.write("");
    delay(500);
    fstring = "U"+String(FWMajorVersion,DEC)+"."+String(FWMinorVersion,DEC);        
    disp.write(fstring);// FW version = MajorVersion.MinorVersion
    delay(500);
    disp.write("");
    delay(500);    
    
    if(i2cadd > 0x0f)
    {
      astring = "A"+String(i2cadd, HEX);    
    }
    else
    {
      astring = "A0"+String(i2cadd, HEX);
    }
        
    disp.write(astring);// direccion i2c
    delay(500);
    disp.write("");
    delay(500);    
    firstboot=1;    
  }
  
  
  // **************************************************************************
  // I2C command processing
  // **************************************************************************
  if(i2c_rcvdone== 1)
  {
    // change I2C address
    if(i2c_buffer[0]==CMD_CHNGADD && i2c_bytesrcvd > 1)
    {
      // Validate new I2C address , write into  EEPROM
      // Reserved adress 0000XXX y 1111XXX
      if( i2c_buffer[1] > 7 &&  i2c_buffer[1] < 120)
        {
          EEPROM.write(EEPROM_I2C_ADD,i2c_buffer[1]); // Required power cycle to take effect
          delay(1000); // just in case!
          blinkrate=5;
        }      
    }
    // change blink rate
    if(i2c_buffer[0]==CMD_BLNKRATE && i2c_bytesrcvd > 1)
    {
      // Validate blink rate
      if( i2c_buffer[1] > 0 &&  i2c_buffer[1] < 10)
        {
           blinkrate = i2c_buffer[1];
        }      
    }    
    // Display received text string
    if(i2c_buffer[0]==CMD_DISPTEXT && i2c_bytesrcvd > 1)
    {
      int steps=0;
      int m=0;
      if(i2c_bytesrcvd > MAX_CHARS)
        steps=MAX_CHARS;
      else
        steps=i2c_bytesrcvd-1;  
      
      dispvalue = "";
      
      while( steps > 0 )
      {
        unsigned char ch=i2c_buffer[m+1]; 
        dispvalue = dispvalue + char(ch);
        steps--;
        m++;
      }  
    }    
    i2c_rcvdone=0;
  }
  
  
  // *******************************************************
  //     Display data on to the 7 segment ( at the selected blink rate! )
  // *******************************************************
  
  if(blinkrate == 0)
  {
    blinkstate = HIGH;
  }
  else
  {
    unsigned long currentMillis = millis();
    unsigned long interval = 500/blinkrate;
 
    if(currentMillis - previousMillis > interval) 
    {
      // store last time display blink
      previousMillis = currentMillis;   
      // if ON then turn OFF and vice-versa:
      if (blinkstate == LOW)
      {
        blinkstate = HIGH;
      }      
      else
      {
        blinkstate = LOW;
      }
    }

  }
  
  
  if(blinkstate == HIGH)
    disp.write(dispvalue);
  else
    disp.write("");
  
  
}

ISR(TIMER1_COMPA_vect)   {
  disp.interruptAction();
}

// function executed everytime a data is received from an I2C master
// function registered as an event, see setup()

void receiveEvent(int howMany)
{
  // First byte is a command: CHANGE I2C ADDRESS, BLINK SPEED, DISPLAY DATA
  
  i2c_bytesrcvd=0;  
  while(Wire.available()) // I2C data available?
  {
    int i2c_byte = Wire.read(); // read byte and write to buffer
    i2c_buffer[i2c_bytesrcvd]=i2c_byte;
    i2c_bytesrcvd++;  
  }
  i2c_rcvdone=1;

}
